// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:wyatt_ui_components/wyatt_ui_components.dart';
import 'package:wyatt_ui_layout/src/core/scaffold_fields_wrapper.dart';
import 'package:wyatt_ui_layout/src/presentation/layouts/structural_layouts/structural_layout.dart';

/// {@template top_bar_layout}
/// An abstract class for creating layouts with a top bar component.
///
/// This class provides a base for creating layouts that include a top bar
/// component, such as an app bar or navigation bar.
///
/// Implementations of this class must provide a concrete implementation of
/// the [appBar] method, which returns the specific top bar component for the
/// given [BuildContext].
///
/// [T] represents the type of the top bar component.
/// {@endtemplate}
abstract class TopBarLayout<T extends TopBarComponent>
    extends StructuralLayout {
  /// {@macro top_bar_layout}
  const TopBarLayout({
    required this.body,
    this.custom,
    this.barId,
    this.height = 60,
    this.scaffoldFieldsWrapper,
    super.key,
  });

  /// The main content of the layout.
  final Widget body;

  /// An optional function that can be used to customize the top bar component.
  /// The function takes the top bar component as an argument and returns
  /// a customized top bar component.
  final T? Function(T?)? custom;

  /// The parameter [barId] enables to specify the particular app bar to use.
  final String? barId;

  /// The height of the top bar.
  final double height;

  /// Returns the top bar component for the given [BuildContext].
  T? appBar(BuildContext context, String? barId);

  /// The [scaffoldFieldsWrapper] is a final variable that serves as a wrapper
  /// for customizing the scaffold.
  /// It allows for tailored modifications and enhancements to the standard
  /// scaffold functionality.
  final ScaffoldFieldsWrapper? scaffoldFieldsWrapper;

  PreferredSize? _buildAppBar(BuildContext context) {
    final topAppBar =
        custom?.call(appBar(context, barId)) ?? appBar(context, barId);

    return topAppBar != null
        ? PreferredSize(
            preferredSize: Size.fromHeight(height),
            child: topAppBar,
          )
        : null;
  }

  @override
  Widget build(BuildContext context) => Scaffold(
        appBar: _buildAppBar(context),
        body: body,
        floatingActionButtonLocation:
            scaffoldFieldsWrapper?.floatingActionButtonLocation,
        floatingActionButtonAnimator:
            scaffoldFieldsWrapper?.floatingActionButtonAnimator,
        persistentFooterButtons: scaffoldFieldsWrapper?.persistentFooterButtons,
        persistentFooterAlignment:
            scaffoldFieldsWrapper?.persistentFooterAlignment ??
                AlignmentDirectional.centerEnd,
        drawer: scaffoldFieldsWrapper?.drawer,
        onDrawerChanged: scaffoldFieldsWrapper?.onDrawerChanged,
        endDrawer: scaffoldFieldsWrapper?.endDrawer,
        onEndDrawerChanged: scaffoldFieldsWrapper?.onEndDrawerChanged,
        bottomSheet: scaffoldFieldsWrapper?.bottomSheet,
        backgroundColor: scaffoldFieldsWrapper?.backgroundColor,
        resizeToAvoidBottomInset:
            scaffoldFieldsWrapper?.resizeToAvoidBottomInset,
        primary: scaffoldFieldsWrapper?.primary ?? true,
        drawerDragStartBehavior:
            scaffoldFieldsWrapper?.drawerDragStartBehavior ??
                DragStartBehavior.start,
        extendBody: scaffoldFieldsWrapper?.extendBody ?? false,
        extendBodyBehindAppBar:
            scaffoldFieldsWrapper?.extendBodyBehindAppBar ?? false,
        drawerScrimColor: scaffoldFieldsWrapper?.drawerScrimColor,
        drawerEdgeDragWidth: scaffoldFieldsWrapper?.drawerEdgeDragWidth,
        drawerEnableOpenDragGesture:
            scaffoldFieldsWrapper?.drawerEnableOpenDragGesture ?? true,
        endDrawerEnableOpenDragGesture:
            scaffoldFieldsWrapper?.endDrawerEnableOpenDragGesture ?? true,
        restorationId: scaffoldFieldsWrapper?.restorationId,
      );
}

/// A concrete implementation of [TopBarLayout] for an app bar.
class TopAppBarLayout extends TopBarLayout<TopAppBarComponent> {
  /// Creates a [TopAppBarLayout] instance.
  ///
  /// [body] represents the main content of the layout.
  /// [custom] is an optional function that can be used to customize
  ///  the top bar component.
  /// [height] represents the height of the top bar.
  const TopAppBarLayout({
    required super.body,
    super.barId,
    super.custom,
    super.height,
    super.scaffoldFieldsWrapper,
    super.key,
  });

  PreferredSize? _buildAppBar(BuildContext context) {
    final appBar = custom?.call(
          context.components.topAppBarComponentOrNull(barId)?.call(),
        ) ??
        context.components.topAppBarComponentOrNull(barId)?.call();

    return appBar != null
        ? PreferredSize(
            preferredSize: Size.fromHeight(height),
            child: appBar,
          )
        : null;
  }

  @override
  TopAppBarComponent? appBar(BuildContext context, String? barId) =>
      context.components.topAppBarComponentOrNull(barId)?.call();
}

/// A concrete implementation of [TopBarLayout] for a navigation bar.
class TopNavigationBarLayout extends TopBarLayout<TopNavigationBarComponent> {
  /// Creates a [TopNavigationBarLayout] instance.
  ///
  /// [body] represents the main content of the layout.
  /// [custom] is an optional function that can be used to customize
  ///  the top bar component.
  /// [height] represents the height of the top bar.
  const TopNavigationBarLayout({
    required super.body,
    super.barId,
    super.custom,
    super.height,
    super.scaffoldFieldsWrapper,
    super.key,
  });

  @override
  TopNavigationBarComponent? appBar(BuildContext context, String? barId) =>
      context.components.topNavigationBarComponentOrNull(barId)?.call();
}
